﻿using log4net;
using Sportradar.SDK.Common.Interfaces;
using Sportradar.SDK.FeedProviders.Lcoo;
using System;
using System.Threading;
using System.Threading.Tasks;

namespace Sportradar.DemoProject
{
    public class LcooModule : IStartable
    {
        private static readonly ILog g_log = LogManager.GetLogger(typeof(LiveScoutModule));
        private readonly string m_feed_name;
        private readonly ILcoo m_lcoo;
        private DateTime m_last_timestamp;

        public LcooModule(ILcoo lcoo, string feed_name)
        {
            m_feed_name = feed_name;
            m_lcoo = lcoo;
            m_lcoo.OnMatch += lcoo_OnMatch;
            m_lcoo.OnOutright += lcoo_OnOutright;
            m_lcoo.OnThreeBall += m_lcoo_OnThreeBall;
        }

        public void Start()
        {
            g_log.InfoFormat("{0} Starting", m_feed_name);
            m_lcoo.Start();
        }

        public void Stop()
        {
            m_lcoo.Stop();
        }

        private void CheckIfOldData(DateTime? timestamp)
        {
            if (timestamp == null)
            {
                return;
            }
            DateTime time = (DateTime)timestamp;
            if (!time.Equals(m_last_timestamp))
            {
                m_last_timestamp = time;
                if (DateTime.UtcNow.Subtract(time) > TimeSpan.FromMinutes(30))
                {
                    //Lets delete queue as data fetching for 60+ messages will take too long (max 6 per minute)
                    Task.Factory.StartNew(() =>
                    {
                        //LCoO HTTP has 10 second access limit
                        Thread.Sleep(TimeSpan.FromSeconds(10));
                        g_log.InfoFormat("{0}: Deleting queue, make sure to call full update from betradar.com", m_feed_name);
                        m_lcoo.ClearQueue();
                    });
                }
                else if (DateTime.UtcNow.Subtract(time) > TimeSpan.FromMinutes(5))
                {
                    g_log.WarnFormat("{0}: Received message with timestamp {1}, is it too old to accept bets?", m_feed_name, timestamp);
                }
            }
        }

        private void lcoo_OnMatch(object sender, MatchEventOdds e)
        {
            CheckIfOldData(e.MatchEntity.MessageTime);
            g_log.InfoFormat("{0}: Received Match with id {1}", m_feed_name, e.MatchEntity.MatchId);
        }

        private void lcoo_OnOutright(object sender, OutrightEventOdds e)
        {
            CheckIfOldData(e.OutrightEntity.MessageTime);
            g_log.InfoFormat("{0}: Received Outright with id {1}", m_feed_name, e.OutrightEntity.Id);
        }

        private void m_lcoo_OnThreeBall(object sender, ThreeBallEventArgs e)
        {
            CheckIfOldData(e.ThreeBallEntity.MessageTime);
            g_log.InfoFormat("{0}: Received ThreeBall with id {1}", m_feed_name, e.ThreeBallEntity.Id);
        }
    }
}